﻿using System;
using System.Collections.Generic;
using DataAccess;
using MbUnit.Framework;
using Rhino.Mocks;

namespace BusinessLogic.Tests
{
    [TestFixture]
    public class OrderControllerTests
    {
        private MockRepository m_Mocks;
        private IOrderRepository m_OrderRepos = null;
        private IPaymentService m_PaymentService = null;
        private Order m_MockedOrder = null;

        [SetUp]
        public void Setup()
        {
            m_Mocks = new MockRepository();
            m_OrderRepos = m_Mocks.CreateMock<IOrderRepository>();
            OrderController.OrderRepository = m_OrderRepos;

            m_PaymentService = m_Mocks.CreateMock<IPaymentService>();
            OrderController.PaymentService = m_PaymentService;

            m_MockedOrder = MockOrder();
            
        }

        private Order MockOrder()
        {
            Order o = new Order();
            o.OrderDate = DateTime.Now;

            Order_Detail or = new Order_Detail();
            or.Order = o;
            
            Product p = new Product();
            p.ProductName = "Test";

            or.Product = p;
            or.UnitPrice = 123;
            or.Quantity = 1;

            return o;
        }

        private Payment CreatePayment()
        {
            Payment p = new Payment();
            p.Amount = 122;
            p.CustomerID = "2134";
            p.ExpiryDate = DateTime.Now;
            p.LongNumberAcrossTheMiddle = 123;

            return p;

        }
        
        private Customer CreateCustomer()
        {
            Customer cust = new Customer();
            cust.CustomerID = "TEST1";
            cust.ContactName = "John Smith";
            cust.CompanyName = "Some Company";

            return cust;
        }
        
        [Test]
        public void TakePayment_ValidInformation_ReturnsTrue()
        {
            //Payment p = new Payment();
            //p.Amount = 122;
            //p.CustomerID = "2134";
            //p.ExpiryDate = DateTime.Now;
            //p.LongNumberAcrossTheMiddle = 123;

            Payment p = CreatePayment();

            using (m_Mocks.Record())
            {
                Expect.Call(m_PaymentService.TakePayment(m_MockedOrder, p)).Return(true);
            }

            using (m_Mocks.Playback())
            {
                bool result = OrderController.TakePayment(m_MockedOrder, p);
                Assert.IsTrue(result);
            }
        }

        [Test]
        public void PlaceOrder_ValidOrderInformation_CallsMethods()
        {
            Payment p = CreatePayment();

            using (m_Mocks.Record())
            {
                Expect.Call(m_PaymentService.TakePayment(m_MockedOrder, p)).Return(true);
                Expect.Call(m_OrderRepos.PlaceOrder(m_MockedOrder)).Return(true);
            }

            using (m_Mocks.Playback())
            {
                bool result = OrderController.PlaceOrder(m_MockedOrder, p);
                Assert.IsTrue(result);
            }
        }

        [Test]
        public void GetOrder_ValidId_CallsMethod()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_OrderRepos.GetOrder(2134)).Return(m_MockedOrder);
            }

            using (m_Mocks.Playback())
            {
                Order o = OrderController.GetOrder(2134);
                Assert.AreEqual(m_MockedOrder, o);
            }
        }

        [Test]
        public void GetOrdersForCustomer_ValidCustomer_CallsMethod()
        {
            Customer c = CreateCustomer();
            using (m_Mocks.Record())
            {
                Expect.Call(m_OrderRepos.GetOrdersForCustomer(c)).Return(new List<Order> { m_MockedOrder });
            }

            using (m_Mocks.Playback())
            {
                List<Order> orders = OrderController.GetOrdersForCustomer(c);

                Assert.AreEqual(1, orders.Count);
                Assert.AreEqual(m_MockedOrder, orders[0]);
            }
        }

        [Test]
        public void GetTotal_3OrderLines_TotalOf3()
        {
            Order o = new Order();
            Order_Detail or = new Order_Detail();
            or.UnitPrice = 100;
            Order_Detail or2 = new Order_Detail();
            or2.UnitPrice = 50;
            Order_Detail or3 = new Order_Detail();
            or3.UnitPrice = 50;

            o.Order_Details.Add(or);
            o.Order_Details.Add(or2);
            o.Order_Details.Add(or3);
            
            decimal actual = o.GetTotal();
            Assert.AreEqual(200, actual);
            
        }
    }
}
