﻿using System;
using System.Collections.Generic;
using BusinessLogic;
using DataAccess;
using MbUnit.Framework;
using Rhino.Mocks;

namespace TestingLinqToSQL.Tests
{
    [TestFixture]
    public class ProductControllerTests
    {
        MockRepository m_Mocks;
        IProductRepository m_ProductRepos;
        Category m_MockedCategory;
        Product m_MockedProduct;

        [SetUp]
        public void Setup()
        {
            m_Mocks = new MockRepository();
            m_ProductRepos = m_Mocks.CreateMock<IProductRepository>();

            //Parameter Injection (IoC)
            ProductController.ProductRepository = m_ProductRepos;

            m_MockedCategory = MockCategory();
            m_MockedProduct = MockProduct();
        }

        [TearDown]
        public void Teardown()
        {
            m_Mocks = null;
        }

        private Category MockCategory()
        {
            Category c = new Category();
            c.CategoryID = 1;
            c.CategoryName = "Software";
            c.Description = "All the latest Microsoft releases.";
            return c;

        }

        private Product MockProduct()
        {
            Product p = new Product();
            p.ProductID = 1;
            p.ProductName = "Visual Studio 2008";
            p.UnitPrice = 1000;
            return p;
        }

        [Test]
        public void GetCategories_NoContent_ReturnEmptyList()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetCategories()).Return(new List<Category>());
            }

            using (m_Mocks.Playback())
            {
                List<Category> categories = ProductController.GetCategories();
                Assert.IsNotNull(categories);
                Assert.AreEqual(0, categories.Count);
            }
        }

        [Test]
        public void GetCategories_SimpleContent_ReturnPopulatedList()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetCategories()).Return(new List<Category> { m_MockedCategory });
            }

            using (m_Mocks.Playback())
            {
                List<Category> categories = ProductController.GetCategories();
                Assert.IsNotNull(categories);
                Assert.AreEqual(1, categories.Count);
                Assert.AreEqual("Software", categories[0].CategoryName);
            }
        }

        [Test]
        public void GetCategoryByName_NameOfValidCategory_ReturnCategoryObject()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetCategory("")).IgnoreArguments().Return(m_MockedCategory);
            }

            using (m_Mocks.Playback())
            {
                Category c = ProductController.GetCategory("Software");
                Assert.IsNotNull(c);
                Assert.AreEqual("Software", c.CategoryName);
                Assert.AreEqual("All the latest Microsoft releases.", c.Description);
            }
        }

        [Test]
        public void GetProductsForCategory_ValidCategory_EmptyList()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetProductsForCategory(m_MockedCategory)).IgnoreArguments().Return(new List<Product>());
            }

            using (m_Mocks.Playback())
            {
                List<Product> products = ProductController.GetProductsForCategory(m_MockedCategory);
                Assert.AreEqual(0, products.Count);
            }
        }

        [Test]
        public void GetProductsForCateogry_ValidCategoryWithProduct_PopulatedList()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetProductsForCategory(m_MockedCategory)).Return(new List<Product> { m_MockedProduct });
            }

            using (m_Mocks.Playback())
            {
                List<Product> products = ProductController.GetProductsForCategory(m_MockedCategory);
                Assert.AreEqual(1, products.Count);
                Assert.AreEqual("Visual Studio 2008", products[0].ProductName);
            }
        }

        [Test]
        public void GetProductByID_ValidProductID_ReturnProductID()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetProduct(1)).Return(m_MockedProduct);
            }

            using (m_Mocks.Playback())
            {
                Product p = ProductController.GetProduct(1);
                Assert.AreEqual(p.ProductName, m_MockedProduct.ProductName);
            }
        }

        [Test]
        public void GetAllProducts_ValidProduct_ReturnPopulatedList()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetAllProducts()).Return(new List<Product> { m_MockedProduct });
            }

            using (m_Mocks.Playback())
            {
                List<Product> p = ProductController.GetAllProducts();
                Assert.AreEqual(1, p.Count);
            }
        }

        [Test]
        public void FindCategoryByID_ValidCategoryID_CategoryObjectReturned()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetCategoryById(1)).Return(m_MockedCategory);
            }

            using (m_Mocks.Playback())
            {
                Category c = ProductController.FindCategoryByID(1);
                Assert.AreEqual(m_MockedCategory.CategoryName, c.CategoryName);
            }
        }

        #region Validation Tests

        [Test]
        [ExpectedArgumentNullException]
        public void GetCategoryByName_StringEmptyName_ExceptionThrown()
        {
            Category c = ProductController.GetCategory(string.Empty);
            Assert.Fail("Should thrown exception");
        }

        [Test]
        [ExpectedArgumentNullException]
        public void GetCategoryByName_StringNull_ExceptionThrown()
        {
            Category c = ProductController.GetCategory(null);
            Assert.Fail("Should thrown exception");
        }

        [Test]
        [ExpectedArgumentNullException]
        public void GetProductByID_InvalidProductID_ThrowException()
        {
            Product p = ProductController.GetProduct(-1);
            Assert.Fail("Should thrown exception");
        }

        [Test]
        [ExpectedArgumentNullException]
        public void GetProductsForCateogry_NullCategory_ThrowException()
        {
            List<Product> products = ProductController.GetProductsForCategory(null);
            Assert.Fail("Should thrown exception");
        }

        [Test]
        [ExpectedException(typeof(Exception))]
        public void GetProductsForCateogry_DatabaseError_ThrowException()
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetProductsForCategory(m_MockedCategory)).Throw(new Exception());
            }

            using (m_Mocks.Playback())
            {
                List<Product> products = ProductController.GetProductsForCategory(null);
                Assert.AreEqual(1, products.Count);
                Assert.AreEqual("Visual Studio 2008", products[0].ProductName);
            }
        }

        #endregion

        #region RowTests

        [RowTest]
        [Row(1)]
        [Row(-1, ExpectedException = typeof(ArgumentNullException))]
        [Row(0, ExpectedException=typeof(ArgumentNullException))]
        //[Row(Int32.MaxValue + 1, ExpectedException = typeof(ArgumentNullException))] Doesn't compile :)
        public void GetProductByID_ProductID(int productId)
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetProduct(productId)).Return(m_MockedProduct);
            }

            using (m_Mocks.Playback())
            {
                Product p = ProductController.GetProduct(productId);
                Assert.AreEqual(p.ProductName, m_MockedProduct.ProductName);
            }
        }


        [RowTest]
        [Row("Software")]
        [Row("", ExpectedException = typeof(ArgumentNullException))]
        [Row(null, ExpectedException = typeof(ArgumentNullException))]
        public void GetCategoryByName_NameOfCategory(string categoryName)
        {
            using (m_Mocks.Record())
            {
                Expect.Call(m_ProductRepos.GetCategory(categoryName)).Return(m_MockedCategory);
            }

            using (m_Mocks.Playback())
            {
                Category c = ProductController.GetCategory(categoryName);

                Assert.IsNotNull(c);
                Assert.AreEqual("Software", c.CategoryName);
                Assert.AreEqual("All the latest Microsoft releases.", c.Description);
            }
        }

        #endregion
    }
}
