﻿using System;
using System.Linq;
using MbUnit.Core.Exceptions;
using MbUnit.Framework;

namespace DataAccess.Tests
{
    static class TestHelper
    {
        //Accessing these via the DataConText to save implementing the methods on the ProductRepository.
        //We could have used ADO.net
        internal static Category InsertCategory()
        {
            Category c = new Category();
            c.CategoryName = "Software";
            c.Description = "All the latest Microsoft releases.";

            TestingApplicationsWithMbUnitDataContext db = new TestingApplicationsWithMbUnitDataContext();
            db.Categories.InsertOnSubmit(c);
            db.SubmitChanges();

            return c;
        }

        internal static Product InsertProduct(Category c)
        {
            TestingApplicationsWithMbUnitDataContext db = new TestingApplicationsWithMbUnitDataContext();

            Product p = new Product();
            p.ProductName = "Visual Studio 2008";
            p.UnitPrice = 1000;
            db.Products.InsertOnSubmit(p);
            db.SubmitChanges();

            db.Categories.Attach(c);
            c.Products.Add(p);
            db.SubmitChanges();

            return p;
        }

        internal static Customer InsertCustomer()
        {
            Customer cust = new Customer();
            cust.CustomerID = "TEST1";
            cust.ContactName = "John Smith";
            cust.CompanyName = "Some Company";

            TestingApplicationsWithMbUnitDataContext db = new TestingApplicationsWithMbUnitDataContext();
            db.Customers.InsertOnSubmit(cust);
            db.SubmitChanges();

            return cust;
        }

        internal static Order InsertOrder()
        {
            Order o = new Order();
            o.OrderDate = DateTime.Now;

            Order_Detail or = new Order_Detail();
            or.Order = o;
            or.Product = InsertProduct(InsertCategory());
            or.UnitPrice = 123;
            or.Quantity = 1;
            
            TestingApplicationsWithMbUnitDataContext db = new TestingApplicationsWithMbUnitDataContext();
            db.Orders.InsertOnSubmit(o);
            db.SubmitChanges();

            return o;
        }

        internal static Order InsertOrderWithCustomer(Customer c)
        {
            Order o = new Order();
            o.Customer = c;
            o.OrderDate = DateTime.Now;

            Order_Detail or = new Order_Detail();
            or.Order = o;
            or.Product = InsertProduct(InsertCategory());
            or.UnitPrice = 123;
            or.Quantity = 1;

            TestingApplicationsWithMbUnitDataContext db = new TestingApplicationsWithMbUnitDataContext();
            db.Customers.Attach(c);
            db.Orders.InsertOnSubmit(o);
            db.Order_Details.InsertOnSubmit(or);
            db.SubmitChanges();

            return o;
        }

        internal static void AreEqual(Category expected, Category actual)
        {
            Assert.AreEqual(expected.CategoryID, actual.CategoryID);
            Assert.AreEqual(expected.CategoryName, actual.CategoryName);
            Assert.AreEqual(expected.Description, actual.Description);
            Assert.AreEqual(expected.Picture, actual.Picture);
        }
    }

    [TestFixture]
    public class TestHelperTests
    {
        [Test]
        [RollBack]
        public void InsertCategory_ValidCategoryObject_UpdatesDatabase()
        {
            Category sample = TestHelper.InsertCategory();

            TestingApplicationsWithMbUnitDataContext db = new TestingApplicationsWithMbUnitDataContext();
            Category c = (from cate in db.Categories
                          where cate.CategoryName == "Software"
                          select cate).Single();

            Assert.IsNotNull(c);
            Assert.AreEqual(sample.CategoryName, c.CategoryName);
            Assert.AreEqual(sample.Description, c.Description);
        }

        [Test]
        [RollBack]
        public void InsertProduct_ValidProductObject_UpdatesDatabase()
        {
            Category testCategory = TestHelper.InsertCategory();
            Product sample = TestHelper.InsertProduct(testCategory);

            TestingApplicationsWithMbUnitDataContext db = new TestingApplicationsWithMbUnitDataContext();
            Product p = (from prod in db.Products
                          where prod.ProductName == "Visual Studio 2008"
                          select prod).Single();

            Assert.IsNotNull(p);
            Assert.AreEqual(testCategory.CategoryID, p.CategoryID);
            Assert.AreEqual(sample.ProductName, p.ProductName);
        }

        [Test]
        [RollBack]
        public void InsertCustomer_ValidCustomerObject_UpdatesDatabase()
        {
            Customer sample = TestHelper.InsertCustomer();

            TestingApplicationsWithMbUnitDataContext db = new TestingApplicationsWithMbUnitDataContext();

            Customer c = (from cust in db.Customers
                         where cust.ContactName == "John Smith"
                         select cust).Single();

            Assert.AreEqual(sample.ContactName, c.ContactName);
            Assert.AreEqual(sample.CompanyName, c.CompanyName);
        }

        [Test]
        [RollBack]
        public void InsertOrder_ValidOrderObject_UpdatesDatabase()
        {
            Order sample = TestHelper.InsertOrder();

            TestingApplicationsWithMbUnitDataContext db = new TestingApplicationsWithMbUnitDataContext();

            Order o = (from orders in db.Orders
                       select orders).Single();

            Order_Detail or = (from oline in db.Order_Details
                               where oline.OrderID == o.OrderID
                               select oline).Single();

            Assert.AreEqual(sample.OrderID, o.OrderID);
            Assert.AreEqual(o.OrderID, or.OrderID);
        }

        [Test]
        [RollBack]
        public void InsertOrderWithCustomer_ValidOrderObject_UpdatesDatabase()
        {
            Customer c = TestHelper.InsertCustomer();
            Order sample = TestHelper.InsertOrderWithCustomer(c);

            TestingApplicationsWithMbUnitDataContext db = new TestingApplicationsWithMbUnitDataContext();

            Order o = (from orders in db.Orders
                       select orders).Single();

            Order_Detail or = (from oline in db.Order_Details
                               where oline.OrderID == o.OrderID
                               select oline).Single();

            Assert.AreEqual(sample.OrderID, o.OrderID);
            Assert.AreEqual(o.OrderID, or.OrderID);
            Assert.IsNotNull(o.CustomerID);
            Assert.AreEqual(c.CustomerID, o.CustomerID);
        }

        [Test]
        public void CategoryAreEqual_TwoEqualCategories_Passes()
        {
            Category c = new Category();
            c.CategoryID = 1;
            c.CategoryName = "Category1";
            c.Description = "This is a test category";

            Category c2 = new Category();
            c2.CategoryID = 1;
            c2.CategoryName = "Category1";
            c2.Description = "This is a test category";

            TestHelper.AreEqual(c, c2);
        }

        [Test]
        [ExpectedException(typeof(NotEqualAssertionException))]
        public void CategoryAreEqual_TwoNotEqualCategories_ThrowsException()
        {
            Category c = new Category();
            c.CategoryID = 1;
            c.CategoryName = "Category1";
            c.Description = "This is a test category";

            Category c2 = new Category();
            c2.CategoryID = 2;
            c2.CategoryName = "Category1";
            c2.Description = "This is a test category";

            TestHelper.AreEqual(c, c2);
        }
    }
}
